/*
 	DMX reciver code for arduino with ATmega 368
 	It is based on Hendrik Hoelschers dmx code from http://www.hoelscher-hi.de/hendrik/
	This code was adapted for the ATmega 8 by big_mark_h
 	And finaly Gustaf Gagge adapted it for ATmega 168 with lots of help from big_mark_h and Grumpy_Mike

	this version codes things a little bit different and includes a polling function, but the main idea is the same.

 	on an arduino MEGA, we can use serial port 1 instead of serial port 0
 	if so, uncomment the next line
*/

//#define DMX_USE_PORT1

#define USEHAMMING			1							// enable or disable the hamming encoding
#define BASECHANNEL 		1							// the start channel of this device
#define CHANNELFORCE 		true					// force the channel to the one set in the define, ignore what is in the EEPROM and write this number in it.
#define CHANNELCOUNT		5 * 16				// on the scanmaster2 controller, this is equal to 5 full scanner fixtures.
#define LEDPIN 					13						// the pin for a LED, if available. undef if not
#define LEDREFRESH			200						// the refresh rate for the LED flash
#define MAXQUEUE 				15						// maximum queue length. 15 is usually more than enough
#define EEPROMADDR			10						// the place where the basechannel is set in EEPROM. This makes it possible to set it through software.
#define SETBUTTONPIN		5							// the pin for the setup button. Active low.
#define SETTIMEOUT			2000					// the timeout to hold the setbutton pin low to engage setup.
#define DATATIMEOUT			3000

#define NOCHANNELFORCE 	false					// basically just for the begin function to set stuff.
#define NOLEDPIN				0
#define NOSETBUTTON			0



#ifndef DMXCLIENT_h
#define DMXCLIENT_h

#if RAMEND <= 0x4FF
#define DMX_MAX_SIZE 128
#else
#define DMX_MAX_SIZE 512
#endif
#define DMXSPEED 250000L
#define DMXPROBE_MAX_TIME 50

#ifndef DMX_USE_PORT1

#define UBRRnH 	UBRR0H
#define UBRRnL 	UBRR0L
#define UCSRnB 	UCSR0B
#define UCSRnC 	UCSR0C
#define RXENn 	RXEN0
#define RXCIEn 	RXCIE0
#define UCSRnA 	UCSR0A
#define UDRn		UDR0
#define FEn			FE0
#define RXCn		RXC0

#else

#define UBRRnH 	UBRR1H
#define UBRRnL 	UBRR1L
#define UCSRnB 	UCSR1B
#define UCSRnC 	UCSR1C
#define RXENn 	RXEN1
#define RXCIEn 	RXCIE1
#define UCSRnA 	UCSR1A
#define UDRn		UDR1
#define FEn			FE1
#define RXCn		RXC1

#endif

enum {RECVSTART, RECVDONE, RECVIDLE, RECVSKIPSTARTBYTE,RECVSAVEDATA};
typedef enum {DMXOFF,DMXPOLL,DMXCONT} DMXMODE;
typedef enum {UARTOFF,UARTRONLY,UARTFULL} UARTMODE;

#define CalcPreScale(B) (((((F_CPU) / 8) / (B)) - 1) / 2)
const int32_t dmxPreScale = CalcPreScale(DMXSPEED); // BAUD prescale factor for DMX speed.

class DMXClientClass {
	public:
		bool dmxData	 				= false;  /* updated by lastssen so we do not have to constantly call this 'expensive' function. */
		bool dmxDataChanged 	= false;
		bool dmxState 				= false;
		bool extSwitch				= false;
		bool ledState 				= false;
  	bool setState					= false;
		
		uint8_t* dmxBuffer;
		uint8_t	 ledMode			= 0;
		volatile uint16_t dmxAddress;
		uint16_t dmxChannelCount;
		uint16_t signalTimeout = 5000;
		
		DMXMODE  dmxMode;
		
		void loop (void);
		bool lastSeen(void);
		bool poll(void);
		void begin(uint8_t baseChannel = BASECHANNEL, uint8_t channelCount = CHANNELCOUNT, bool forceChannel = CHANNELFORCE, uint8_t ledPin = LEDPIN, uint8_t setButtonPin = SETBUTTONPIN, uint16_t eepromAddress = EEPROMADDR, DMXMODE dmxMode=DMXMODE::DMXCONT);		
		void receiver(void);
		void updateLed(void);
		void checkSetButton(void);
		
		uint8_t readChannel(uint16_t channel);
		
  private:
  	
  	bool buttonCurrentState 				= false;
  	bool buttonPrevState						= false;
  	volatile bool dmxSaveSetting		= false;
  	
  	uint8_t ledPin											= 0;
  	uint8_t setButtonPin								= 0;
  	volatile uint8_t  bufferState 			= RECVIDLE;
  	volatile uint8_t channelScanCount		= 0;
		volatile uint8_t lastChannelScan		= 0;
		volatile uint8_t lastChannelScanVal	= 0;
  	
		uint16_t dmxCountdown;
		uint16_t dmxCurrChannel;
		uint16_t eepromAddress					= EEPROMADDR;
		uint16_t ledTimeout 						= 0;
		volatile uint16_t dmxBuffPos 		= 0;
		
  	unsigned long lastDmxPacketTime	= 0;
  	unsigned long lastLedTime				= 0;
  	unsigned long buttonDebounce 		= 0;
  	unsigned long buttonTimeout			= 0;

  	void setMode(UARTMODE uartMode = UARTMODE::UARTRONLY);
};

#endif
