#ifndef MDCCLIENT_H
#define MDCCLIENT_H

#define SERIALMON				1
#define USEHAMMING			1							// enable or disable the hamming encoding
#define BASECHANNEL 		1							// the start channel of this device
#define CHANNELFORCE 		true					// force the channel to the one set in the define, ignore what is in the EEPROM and write this number in it.
#define CHANNELCOUNT 		1 						// one channel, define as many as needed
#define LEDPIN 					13						// the pin for a LED, if available. undef if not
#define LEDREFRESH			200						// the refresh rate for the LED flash
#define MAXQUEUE 				15						// maximum queue length. 15 is usually more than enough
#define EEPROMADDR			1							// the place where the basechannel is set in EEPROM. This makes it possible to set it through software.
#define SETBUTTON				5							// the pin for the setup button. Active low.
#define SETTIMEOUT			2000					// the timeout to hold the setbutton pin low to engage setup.

#define NOCHANNELFORCE 	false					// basically just for the begin function to set stuff.
#define NOLEDPIN				0
#define NOSETBUTTON			0

 
#if !(defined(__AVR_ATtiny85__)) && !(defined(__AVR_ATtiny45__)) && !(defined(__AVR_ATmega328P)) && !(defined(__AVR_ATmega328)) && !(defined(ARDUINO_AVR_NANO))
#error no supported AVR found.
#endif

#include "BlockTimes.h"
#include "mdcPkgManager.h"

#ifdef USEHAMMING
#include "HammingCodes.h"
#endif


#if defined(__AVR_ATtiny85__) || defined(__AVR_ATtiny45__)

#undef SERIALMON
#define TINY

// make sure to disable the reset on the attiny, else set ledpin to 0
#define LEDPIN 					5
#define SETBUTTON				3

#endif

class mdcClientClass {
	public:
		
		uint8_t myBaseChannel 				= 0;
		uint8_t	localChannel;
		uint8_t mdcValue;
	
		unsigned long lastInt0Update 	= 0;
		
		void begin (uint8_t baseChannel = BASECHANNEL, uint8_t channelCountIn = CHANNELCOUNT, bool forceChannel = CHANNELFORCE, uint8_t ledPinIn = LEDPIN, uint8_t setButtonPinIn = SETBUTTON, uint16_t eepromAddress = EEPROMADDR );
		bool update (void);
		void updateLed (void);
		void int0handler (void);
		void int0scanner (void);
		void check_settingbutton ( void );
		void check_channelset ( uint8_t newchannel, uint8_t newvalue );
		void resetBuffer (void);
		void swapTriggerEdge (void);
	private:
		
		bool hasUpdates = false;
		bool ledState 									= false;
		bool setButtonState 						= false;
		bool buttonCurrentState;
		bool buttonPrevState						= false;
		bool bit												= false;
		bool dataerror									= false;
		bool validPackage								= false;
		volatile bool signalvalid				= false;
				
		uint8_t ledPin 									= 0;
		uint8_t setButtonPin						= 0;
		uint8_t ledMode									=	0;
		uint8_t channelCount						= 0;
		uint8_t channelScanCount				= 0;
		uint8_t lastChannelScan					= 0;
		uint8_t lastChannelScanVal			= 0;
		uint8_t errorcount							= 0;
		volatile uint8_t edgetrigger		= RISING;
		
		uint8_t queuePtr 								= 0;
		uint8_t leftPtr 								= 0;		
		uint8_t mdcChannel;
		uint8_t modeBits;
		
		uint16_t ledTimeout							= 1000;
		uint16_t intCounter;
		
		uint32_t bitStream;
		
		unsigned long prevUsec					= 0;
		unsigned long currentUsec 			= 0;
		unsigned long timeDiff 					= 0;
		unsigned long ledUpdate 				= 0;
		unsigned long buttonDebounce		= 0;
		unsigned long buttontimeout			= 0;
		
		PKGSTATE pkgState		 						= PKGSTART;
		mdcClientPkg queue[MAXQUEUE];
		
/*#ifdef USEEDGESCAN		
		// stuff for scanning the current polarity.
		volatile bool skipme						= false;
		volatile bool signalvalid 			= false;
		volatile uint8_t	validcount 		= 0;
		volatile uint8_t udiffidx				= 0;
		volatile uint8_t edgetrigger		= RISING;
		volatile uint16_t udiff[2]			= {0,0};
#endif*/
		
};



#endif