#include "mdcPkgManager.h"
#include "HammingCodes.h"

static mdcPkgMessage tempMessage;
static mdcPkgMessage* tempMsgPtr;
static uint8_t tempnr;
static uint8_t tempidx;
static bool tempbool;

/*********************************************************************
	initialise the packetmanager
**********************************************************************/

void mdcPkgManagerClass::begin(uint16_t channel=PKGSTARTCHANNEL,bool useHamming = false,uint16_t channelcount=PKGBUFFERSIZE) {
	
	this->channelMax = channelcount;
	this->messageBuffer = (mdcPkgMessage*) calloc(channelcount, sizeof(mdcPkgMessage));
	this->hammingCode = useHamming;
}

/*********************************************************************
	return the current modebits. For now, only mode 0 is supported.
**********************************************************************/
uint8_t mdcPkgManagerClass::getCurrentModeBits (void) {
	return 0;
}

/*********************************************************************
	return the current bitstream
**********************************************************************/
uint32_t mdcPkgManagerClass::getCurrentBitstream (void) {
	uint8_t tempretrans = this->messageBuffer[this->currentSlot].retrans;
	
	if (! tempretrans) {
		if (this->messageBuffer[this->tailSlot].retrans == 0) {
			if (++this->tailSlot > this->headSlot) {
				this->tailSlot = this->headSlot;
			}
		}
		return 0;
	}
	
#ifndef DEBUGRETRANS
	tempretrans--;
	this->messageBuffer[this->currentSlot].retrans = tempretrans;
	
#endif	 

#ifdef DEBUGBUFFER
	this->currentSlot = 0;
#endif
	
	return this->messageBuffer[this->currentSlot].bitStream;
}

/*********************************************************************
	advance the current slot counter 1, and if overrun, reset to beginning of the active slots.
**********************************************************************/
void mdcPkgManagerClass::advanceSlot (void) {
	
	#ifdef DEBUGBUFFER
		this->currentSlot = 0;
		return;
	#endif
	
	if (this->currentSlot == this->headSlot) { 
		// previous one was headSlot, rotate back to tail.
		this->currentSlot = this->tailSlot;
		return;
	}
		
	if (++this->currentSlot == this->channelMax) { 
		this->currentSlot = 0;
	}	
}
	
/*********************************************************************
	add a message in the buffer.
**********************************************************************/
void mdcPkgManagerClass::addMessage(mdcPkgMessage message) {
	uint8_t arraypointer = this->headSlot;
	
	tempidx = this->tailSlot;
	tempbool = false; // denotes if we found an active slot.
	
	while (tempidx != this->headSlot) {
		if (++tempidx == this->channelMax) {
			tempidx = 0;
		}

		if (this->messageBuffer[tempidx].channelId == message.channelId) {
			tempbool = true;
			arraypointer = tempidx;
		}
	}
	
	if (! tempbool) {
		if (++this->headSlot >= this->channelMax) {
			this->headSlot = 0;
		}

#ifdef DEBUGBUFFER
		this->headSlot = 0;
#endif

		arraypointer = this->headSlot;
	}
	
	memcpy(&this->messageBuffer[arraypointer],&message,sizeof(mdcPkgMessage));
	
}

/*********************************************************************
	create a packetmessage from the given channel and value. set addMessage to true if it has to be added to the buffer.
**********************************************************************/
mdcPkgMessage mdcPkgManagerClass::createMessage (uint8_t channelNumber=0 , uint8_t channelValue=0 , bool addMessage = true) {
	
	tempMessage = {channelNumber,channelValue,0,0};

if (this->hammingCode) {
	tempMessage.hammingChannelL = pgm_read_byte(hammingTX + (channelNumber & 0x0F));
	tempMessage.hammingChannelH = pgm_read_byte(hammingTX + (channelNumber >> 4));;
	tempMessage.hammingValueL = pgm_read_byte(hammingTX + (channelValue & 0x0F));;
	tempMessage.hammingValueH = pgm_read_byte(hammingTX + (channelValue >> 4));;
} else {
	tempMessage.hammingChannelH = 0;
	tempMessage.hammingChannelL = channelNumber & 0xFF;
	tempMessage.hammingValueH = 0;
	tempMessage.hammingValueL = channelValue & 0xFF;
}

	tempMessage.retrans = 2;
	//Serial.print(F("Stored: "));
	//Serial.println(tempMessage.bitStream);
	
	if (addMessage) {
		this->addMessage(tempMessage);
	}
	return tempMessage;
}
	