#ifndef mdcPkgManager_h
#define mdcPkgManager_h
#include "Arduino.h"

/*
*
* these routines merely define a cyclic buffer for sending digital blocks.
*
*/
//#define DEBUGBUFFER			// only use 1 package, at position 0. This means only 1 package can be sent, new ones will overwrite this data.
//#define DEBUGRETRANS		// used to disable the deduction of retransmission. this will overflow the buffer if not used with debugbuffer.

#define PKGRETRANSMAX		3
#define PKGBUFFERSIZE		100
#define PKGSTARTCHANNEL	1

#define BITS 32

#if BITS == 8
#define BITMASK 0x80
#endif

#if BITS == 16
#define BITMASK 0x8000
#endif

#if BITS == 24
#define BITMASK 0x800000
#endif

#if BITS == 32
#define BITMASK 0x80000000
#endif

typedef struct {
	uint8_t	channelId;
	uint8_t	channelValue;
	
	struct {
		unsigned retrans		: 2;
		unsigned modebits		: 2;
		unsigned unused			: 4;
	};
	
	union {
		uint32_t bitStream;
		struct {										// yes, it looks strange, but at the receiver, it should be the other way around, so we can left shift the bits in.
			uint8_t hammingValueL;
			uint8_t	hammingValueH;
			uint8_t	hammingChannelL;
			uint8_t hammingChannelH;
		};
	};
} mdcPkgMessage;

typedef struct {
	uint8_t modebits;
	
	union {
		uint32_t bitStream;
		struct {
			uint8_t hammingValueL;
			uint8_t	hammingValueH;
			uint8_t	hammingChannelL;
			uint8_t hammingChannelH;
		};
	};
} mdcClientPkg;

class mdcPkgManagerClass {
  public:
  	volatile mdcPkgMessage*	messageBuffer;
  	uint8_t 				channelMax;
  	uint8_t					tailSlot = 0;
  	uint8_t					headSlot = 0;
  	uint8_t					currentSlot = 0;
  	bool						started = false;
  	bool 						hammingCode = false;
  	
  	void begin(uint16_t channel=PKGSTARTCHANNEL,bool useHamming = false, uint16_t channelcount=PKGBUFFERSIZE);
  	void advanceSlot(void);
  	
  	uint8_t  getCurrentModeBits(void);
  	uint32_t getCurrentBitstream(void);
  	
		void addMessage(mdcPkgMessage message);
		mdcPkgMessage createMessage(uint8_t channelNumber=0, uint8_t channelValue=0, bool addMessage = true);
		
		
  private:
  	
  	
};


#endif